package io.gitlab.jfronny.commons.concurrent;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class AsyncRequestState {
    private final Lock lock = new ReentrantLock();

    private boolean isRunning = false;
    private boolean isScheduled = false;


    public ScanFinishedResponse emitFinished() {
        lock.lock();
        try {
            if (isScheduled) {
                isScheduled = false;
                return new ScanFinishedResponse(true);
            } else {
                isRunning = false;
                return new ScanFinishedResponse(false);
            }
        } finally {
            lock.unlock();
        }
    }

    public RequestScanResponse request() {
        lock.lock();
        try {
            if (isRunning) {
                isScheduled = true;
                return new RequestScanResponse(false);
            } else {
                isRunning = true;
                return new RequestScanResponse(true);
            }
        } finally {
            lock.unlock();
        }
    }

    public void cancel() {
        lock.lock();
        try {
            isScheduled = false;
        } finally {
            lock.unlock();
        }
    }

    public record ScanFinishedResponse(boolean shouldContinue) {}
    public record RequestScanResponse(boolean shouldStart) {}
}
